# Generated by Django 2.2.8 on 2019-12-13 16:29

import jsonfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ListingCache",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("app", models.CharField(max_length=30)),
                ("searcher_name", models.CharField(max_length=50)),
                ("search_token", models.CharField(max_length=50)),
                ("path", models.CharField(default="", max_length=500)),
                ("listing", jsonfield.fields.JSONField()),
                ("created", models.DateTimeField(auto_now_add=True)),
            ],
            options={
                "unique_together": {("app", "searcher_name", "search_token", "path")},
            },
        ),
    ]
