# Generated by Django 2.2.8 on 2019-12-13 16:29

import jsonfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="SearchQueryCache",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("query_hash", models.CharField(max_length=50, unique=True)),
                ("query", jsonfield.fields.JSONField()),
                ("last_used", models.DateTimeField(auto_now=True)),
            ],
        ),
    ]
