# Generated by Django 2.2.8 on 2020-03-31 08:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('metadata_embedded', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='embeddedinfo',
            name='group',
            field=models.CharField(max_length=300, null=True),
        ),
        migrations.AddField(
            model_name='embeddedinfo',
            name='mediainfo_audio',
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='embeddedinfo',
            name='mediainfo_dual_audio',
            field=models.BooleanField(default=False),
        ),
    ]
